@@ -71,6 +71,8 @@ public:
 
     HandleType targetHandleType() const;
     uint targetHandle() const;
+    QString targetId() const;
+    ContactPtr targetContact() const;
 
     bool isRequested() const;
     ContactPtr initiatorContact() const;
