@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2010 Nokia Corporation
+ * @copyright Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -133,10 +134,16 @@ public:
     }
 
 private Q_SLOTS:
-    void gotMainProperties(QDBusPendingCallWatcher *);
+    void gotMainProperties(QDBusPendingCallWatcher *watcher);
+    void gotAvatarsProperties(QDBusPendingCallWatcher *watcher);
+    void gotPresenceProperties(QDBusPendingCallWatcher *watcher);
 
 private:
     static void introspectMain(ProtocolWrapper *self);
+    void introspectAvatars();
+    void introspectPresence();
+
+    void continueIntrospection();
 
     void fillRCCs();
     bool receiveProperties(const QVariantMap &props);
@@ -144,6 +151,7 @@ private:
     ReadinessHelper *mReadinessHelper;
     ProtocolInfo mInfo;
     QVariantMap mImmutableProps;
+    QQueue<void (ProtocolWrapper::*)()> introspectQueue;
 };
 
 } // Tp
