@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright © 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright © 2009 Nokia Corporation
+ * @copyright Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,5 +25,8 @@
 #include "TelepathyQt4/_gen/future-channel-body.hpp"
 #include "TelepathyQt4/_gen/future-channel.moc.hpp"
 
+#include "TelepathyQt4/_gen/future-channel-dispatcher-body.hpp"
+#include "TelepathyQt4/_gen/future-channel-dispatcher.moc.hpp"
+
 #include "TelepathyQt4/_gen/future-misc-body.hpp"
 #include "TelepathyQt4/_gen/future-misc.moc.hpp"
