@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2010 Nokia Corporation
+ * @copyright Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,6 +27,8 @@
 #error IN_TELEPATHY_QT4_HEADER
 #endif
 
+#include <TelepathyQt4/AvatarSpec>
+#include <TelepathyQt4/PresenceSpec>
 #include <TelepathyQt4/Types>
 
 #include <QMetaType>
@@ -54,6 +57,8 @@ public:
     QString iconName(const QString &protocol) const;
     RequestableChannelClassList requestableChannelClasses(
             const QString &protocol) const;
+    PresenceSpecList allowedPresenceStatuses(const QString &protocol) const;
+    AvatarSpec avatarRequirements(const QString &protocol) const;
 
     static QVariant::Type variantTypeFromDBusSignature(
             const QString &dbusSignature);
