@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009 Nokia Corporation
+ * @copyright Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -55,17 +56,17 @@ protected:
             const Feature &coreFeature = OutgoingFileTransferChannel::FeatureCore);
 
 private Q_SLOTS:
-    void onProvideFileFinished(Tp::PendingOperation *op);
+    TELEPATHY_QT4_NO_EXPORT void onProvideFileFinished(Tp::PendingOperation *op);
 
-    void onSocketConnected();
-    void onSocketDisconnected();
-    void onSocketError(QAbstractSocket::SocketError error);
-    void onInputAboutToClose();
-    void doTransfer();
+    TELEPATHY_QT4_NO_EXPORT void onSocketConnected();
+    TELEPATHY_QT4_NO_EXPORT void onSocketDisconnected();
+    TELEPATHY_QT4_NO_EXPORT void onSocketError(QAbstractSocket::SocketError error);
+    TELEPATHY_QT4_NO_EXPORT void onInputAboutToClose();
+    TELEPATHY_QT4_NO_EXPORT void doTransfer();
 
 private:
-    void connectToHost();
-    void setFinished();
+    TELEPATHY_QT4_NO_EXPORT void connectToHost();
+    TELEPATHY_QT4_NO_EXPORT void setFinished();
 
     struct Private;
     friend struct Private;
