@@ -223,6 +223,17 @@ PendingChannel::PendingChannel(const AccountPtr &account,
             SLOT(onAccountCreateChannelFinished(Tp::PendingOperation*)));
 }
 
+/**
+ * Construct a new PendingChannel object that always fails.
+ *
+ * \param errorName The name of a D-Bus error.
+ * \param errorMessage The error message.
+ */
+PendingChannel::PendingChannel(const QString &errorName, const QString &errorMessage)
+    : PendingOperation(ConnectionPtr()), mPriv(new Private)
+{
+    setFinishedWithError(errorName, errorMessage);
+}
 /**
  * Class destructor.
  */
@@ -311,8 +322,8 @@ uint PendingChannel::targetHandle() const
  * If this channel request has finished, return the immutable properties of
  * the resulting channel. Otherwise, return an empty map.
  *
- * The keys and values in this map are defined by the Telepathy D-Bus
- * specification, or by third-party extensions to that specification.
+ * The keys and values in this map are defined by the \telepathy_spec,
+ * or by third-party extensions to that specification.
  * These are the properties that cannot change over the lifetime of the
  * channel; they're announced in the result of the request, for efficiency.
  * This map should be passed to the constructor of Channel or its subclasses
