@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008 Nokia Corporation
+ * @copyright Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -55,16 +56,16 @@ public:
     ContactAttributesMap attributes() const;
 
 private Q_SLOTS:
-    void onCallFinished(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void onCallFinished(QDBusPendingCallWatcher *watcher);
 
 private:
     friend class ConnectionLowlevel;
 
-    PendingContactAttributes(const ConnectionPtr &connection,
+    TELEPATHY_QT4_NO_EXPORT PendingContactAttributes(const ConnectionPtr &connection,
             const UIntList &handles,
             const QStringList &interfaces, bool reference);
 
-    void failImmediately(const QString &error, const QString &errorMessage);
+    TELEPATHY_QT4_NO_EXPORT void failImmediately(const QString &error, const QString &errorMessage);
 
     struct Private;
     friend struct Private;
