@@ -77,6 +77,8 @@ class OutgoingStreamTubeChannel;
 class Profile;
 class ProfileManager;
 class RoomListChannel;
+class SimpleObserver;
+class SimpleCallObserver;
 class SimpleTextObserver;
 class StreamedMediaChannel;
 class StreamedMediaStream;
@@ -129,6 +131,8 @@ typedef SharedPtr<OutgoingStreamTubeChannel> OutgoingStreamTubeChannelPtr;
 typedef SharedPtr<Profile> ProfilePtr;
 typedef SharedPtr<ProfileManager> ProfileManagerPtr;
 typedef SharedPtr<RoomListChannel> RoomListChannelPtr;
+typedef SharedPtr<SimpleObserver> SimpleObserverPtr;
+typedef SharedPtr<SimpleCallObserver> SimpleCallObserverPtr;
 typedef SharedPtr<SimpleTextObserver> SimpleTextObserverPtr;
 typedef SharedPtr<StreamedMediaChannel> StreamedMediaChannelPtr;
 typedef SharedPtr<StreamedMediaStream> StreamedMediaStreamPtr;
