@@ -44,11 +44,14 @@ protected Q_SLOTS:
     void onContactRemovedFromGroup(const QString &group);
     void expectConnInvalidated();
     void expectContact(Tp::PendingOperation*);
+    void exitOnStateSuccess(Tp::ContactListState);
 
 private Q_SLOTS:
     void initTestCase();
     void init();
 
+    void testGroupsAfterStateChange();
+    void testIntrospectAfterStateChange();
     void testRosterGroups();
     void testNotADeathTrap();
 
@@ -137,6 +140,15 @@ void TestConnRosterGroups::expectContact(Tp::PendingOperation *op)
     mLoop->exit(0);
 }
 
+void TestConnRosterGroups::exitOnStateSuccess(Tp::ContactListState state)
+{
+    qDebug() << "got contact list state" << state;
+
+    if (state == ContactListStateSuccess) {
+        mLoop->exit(0);
+    }
+}
+
 void TestConnRosterGroups::initTestCase()
 {
     initTestCaseImpl();
@@ -177,6 +189,171 @@ void TestConnRosterGroups::init()
     mConnInvalidated = false;
 }
 
+void TestConnRosterGroups::testGroupsAfterStateChange()
+{
+    // Create a conn and make the roster groups related features ready
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
+
+    ContactManagerPtr contactManager = mConn->contactManager();
+
+    Features features = Features() << Connection::FeatureRoster << Connection::FeatureRosterGroups;
+    QVERIFY(connect(mConn->becomeReady(features),
+            SIGNAL(finished(Tp::PendingOperation*)),
+            SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
+    QCOMPARE(mConn->isReady(Connection::FeatureRoster), true);
+    QCOMPARE(mConn->isReady(Connection::FeatureRosterGroups), true);
+
+    // Now start connecting it, and wait for the ContactManager state to turn to Success
+    QVERIFY(connect(contactManager.data(),
+                    SIGNAL(stateChanged(Tp::ContactListState)),
+                    SLOT(exitOnStateSuccess(Tp::ContactListState))));
+
+    mConn->lowlevel()->requestConnect();
+
+    QCOMPARE(mLoop->exec(), 0);
+    QCOMPARE(static_cast<uint>(contactManager->state()),
+             static_cast<uint>(ContactListStateSuccess));
+
+    // The conn should be valid and have the roster groups features ready when it emits Success
+    QVERIFY(mConn->isValid());
+    QCOMPARE(mConn->isReady(Connection::FeatureRoster), true);
+    QCOMPARE(mConn->isReady(Connection::FeatureRosterGroups), true);
+
+    // We should have all the group data downloaded now, check for that
+    QStringList expectedGroups;
+    expectedGroups << QLatin1String("Cambridge") << QLatin1String("Francophones")
+        << QLatin1String("Montreal");
+    expectedGroups.sort();
+    QStringList groups = contactManager->allKnownGroups();
+    groups.sort();
+    QCOMPARE(groups, expectedGroups);
+
+    // Cambridge
+    {
+        QStringList expectedContacts;
+        expectedContacts << QLatin1String("geraldine@example.com")
+            << QLatin1String("helen@example.com")
+            << QLatin1String("guillaume@example.com")
+            << QLatin1String("sjoerd@example.com");
+        expectedContacts.sort();
+        QStringList contacts;
+        Q_FOREACH (const ContactPtr &contact, contactManager->groupContacts(QLatin1String("Cambridge"))) {
+            contacts << contact->id();
+        }
+        contacts.sort();
+        QCOMPARE(contacts, expectedContacts);
+    }
+
+    // Francophones
+    {
+        QStringList expectedContacts;
+        expectedContacts << QLatin1String("olivier@example.com")
+            << QLatin1String("geraldine@example.com")
+            << QLatin1String("guillaume@example.com");
+        expectedContacts.sort();
+        QStringList contacts;
+        Q_FOREACH (const ContactPtr &contact, contactManager->groupContacts(QLatin1String("Francophones"))) {
+            contacts << contact->id();
+        }
+        contacts.sort();
+        QCOMPARE(contacts, expectedContacts);
+    }
+
+    // Montreal
+    {
+        QStringList expectedContacts;
+        expectedContacts << QLatin1String("olivier@example.com");
+        expectedContacts.sort();
+        QStringList contacts;
+        Q_FOREACH (const ContactPtr &contact, contactManager->groupContacts(QLatin1String("Montreal"))) {
+            contacts << contact->id();
+        }
+        contacts.sort();
+        QCOMPARE(contacts, expectedContacts);
+    }
+}
+
+void TestConnRosterGroups::testIntrospectAfterStateChange()
+{
+    // Create a conn and make the roster feature ready
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
+
+    ContactManagerPtr contactManager = mConn->contactManager();
+
+    Features features = Features() << Connection::FeatureRoster;
+    QVERIFY(connect(mConn->becomeReady(features),
+            SIGNAL(finished(Tp::PendingOperation*)),
+            SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
+    QCOMPARE(mConn->isReady(Connection::FeatureRoster), true);
+    QCOMPARE(mConn->isReady(Connection::FeatureRosterGroups), false);
+
+    // Now start connecting it, and wait for the ContactManager state to turn to Success
+    QVERIFY(connect(contactManager.data(),
+                    SIGNAL(stateChanged(Tp::ContactListState)),
+                    SLOT(exitOnStateSuccess(Tp::ContactListState))));
+
+    mConn->lowlevel()->requestConnect();
+
+    QCOMPARE(mLoop->exec(), 0);
+    QCOMPARE(static_cast<uint>(contactManager->state()),
+             static_cast<uint>(ContactListStateSuccess));
+
+    // The conn should be valid and have the roster feature ready when it emits Success, but not
+    // RosterGroups because we didn't request it
+    QVERIFY(mConn->isValid());
+    QCOMPARE(mConn->isReady(Connection::FeatureRoster), true);
+    QCOMPARE(mConn->isReady(Connection::FeatureRosterGroups), false);
+
+    // We should have roster contacts now, but no groups
+    QVERIFY(!contactManager->allKnownContacts().isEmpty());
+    QVERIFY(contactManager->allKnownGroups().isEmpty());
+
+    // Make RosterGroups ready too
+    features = Features() << Connection::FeatureRosterGroups;
+    QVERIFY(connect(mConn->becomeReady(features),
+            SIGNAL(finished(Tp::PendingOperation*)),
+            SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
+    QCOMPARE(mConn->isReady(Connection::FeatureRoster), true);
+    QCOMPARE(mConn->isReady(Connection::FeatureRosterGroups), true);
+
+    // We should still have the contacts, and the state should be success
+    QVERIFY(!contactManager->allKnownContacts().isEmpty());
+    QCOMPARE(static_cast<uint>(contactManager->state()),
+             static_cast<uint>(ContactListStateSuccess));
+
+    // We should have all the group data downloaded now, check for that
+    QStringList expectedGroups;
+    expectedGroups << QLatin1String("Cambridge") << QLatin1String("Francophones")
+        << QLatin1String("Montreal");
+    expectedGroups.sort();
+    QStringList groups = contactManager->allKnownGroups();
+    groups.sort();
+    QCOMPARE(groups, expectedGroups);
+
+    // Cambridge
+    {
+        QStringList expectedContacts;
+        expectedContacts << QLatin1String("geraldine@example.com")
+            << QLatin1String("helen@example.com")
+            << QLatin1String("guillaume@example.com")
+            << QLatin1String("sjoerd@example.com");
+        expectedContacts.sort();
+        QStringList contacts;
+        Q_FOREACH (const ContactPtr &contact, contactManager->groupContacts(QLatin1String("Cambridge"))) {
+            contacts << contact->id();
+        }
+        contacts.sort();
+        QCOMPARE(contacts, expectedContacts);
+    }
+}
+
 void TestConnRosterGroups::testRosterGroups()
 {
     mConn = Connection::create(mConnName, mConnPath,
@@ -198,6 +375,9 @@ void TestConnRosterGroups::testRosterGroups()
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(features), true);
 
+    QCOMPARE(static_cast<uint>(mConn->contactManager()->state()),
+             static_cast<uint>(ContactListStateSuccess));
+
     ContactManagerPtr contactManager = mConn->contactManager();
 
     QStringList expectedGroups;
@@ -643,28 +823,21 @@ void TestConnRosterGroups::cleanup()
 {
     mContact.reset();
 
-    if (mConn) {
-        if (mConn->status() != ConnectionStatusDisconnected) {
-            // Disconnect and wait for the readiness change
-            QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
-                        SIGNAL(finished(Tp::PendingOperation*)),
-                        SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-            QCOMPARE(mLoop->exec(), 0);
+    if (mConn && mConn->requestedFeatures().contains(Connection::FeatureCore)) {
+        QVERIFY(mConnService != NULL);
+
+        if (TP_BASE_CONNECTION(mConnService)->status != TP_CONNECTION_STATUS_DISCONNECTED) {
+            tp_base_connection_change_status(TP_BASE_CONNECTION(mConnService),
+                    TP_CONNECTION_STATUS_DISCONNECTED,
+                    TP_CONNECTION_STATUS_REASON_REQUESTED);
         }
 
-        if (mConn->isValid()) {
-            QVERIFY(connect(mConn.data(),
-                            SIGNAL(invalidated(Tp::DBusProxy *,
-                                               const QString &, const QString &)),
-                            SLOT(expectConnInvalidated())));
-
-            while (!mConnInvalidated) {
-                mLoop->processEvents();
-            }
+        while (mConn->isValid()) {
+            mLoop->processEvents();
         }
 
-        mConn.reset();
     }
+    mConn.reset();
 
     if (mConnService != 0) {
         g_object_unref(mConnService);
