@@ -1,28 +1,18 @@
-#include <QtCore/QDebug>
-#include <QtCore/QTimer>
+#include <tests/lib/test.h>
 
-#include <QtDBus/QtDBus>
+#include <tests/lib/glib-helpers/test-conn-helper.h>
 
-#include <QtTest/QtTest>
-
-#define TP_QT4_ENABLE_LOWLEVEL_API
+#include <tests/lib/glib/contactlist2/conn.h>
 
 #include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
-#include <TelepathyQt4/ConnectionLowlevel>
 #include <TelepathyQt4/Contact>
 #include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/ContactManager>
-#include <TelepathyQt4/PendingChannel>
 #include <TelepathyQt4/PendingContacts>
-#include <TelepathyQt4/PendingReady>
-#include <TelepathyQt4/Debug>
 
 #include <telepathy-glib/debug.h>
 
-#include <tests/lib/glib/contactlist2/conn.h>
-#include <tests/lib/test.h>
-
 using namespace Tp;
 
 class TestConnRoster : public Test
@@ -31,12 +21,16 @@ class TestConnRoster : public Test
 
 public:
     TestConnRoster(QObject *parent = 0)
-        : Test(parent), mConnService(0)
+        : Test(parent), mConn(0),
+          mBlockingContactsFinished(false), mHowManyKnownContacts(0),
+          mGotPresenceStateChanged(false), mGotPPR(false)
     { }
 
 protected Q_SLOTS:
-    void expectConnInvalidated();
-    void expectPendingContactsFinished(Tp::PendingOperation *);
+    void expectBlockingContactsFinished(Tp::PendingOperation *op);
+    void expectBlockStatusChanged(bool blocked);
+    void expectBlockedContactsChanged(const Tp::Contacts &added, const Tp::Contacts &removed,
+            const Tp::Channel::GroupMemberChangeDetails &details);
     void expectPresencePublicationRequested(const Tp::Contacts &);
     void expectPresenceStateChanged(Tp::Contact::PresenceState);
     void expectAllKnownContactsChanged(const Tp::Contacts &added, const Tp::Contacts &removed,
@@ -52,46 +46,57 @@ private Q_SLOTS:
     void cleanupTestCase();
 
 private:
-    QString mConnName, mConnPath;
-    ExampleContactListConnection *mConnService;
-    ConnectionPtr mConn;
-    QList<ContactPtr> mContacts;
+    TestConnHelper *mConn;
+    QSet<QString> mContactsExpectingBlockStatusChange;
+    bool mBlockingContactsFinished;
     int mHowManyKnownContacts;
     bool mGotPresenceStateChanged;
     bool mGotPPR;
 };
 
-void TestConnRoster::expectConnInvalidated()
+void TestConnRoster::expectBlockingContactsFinished(Tp::PendingOperation *op)
 {
-    mLoop->exit(0);
+    TEST_VERIFY_OP(op);
+
+    qDebug() << "blocking contacts finished";
+    mBlockingContactsFinished = true;
+
+    if (mContactsExpectingBlockStatusChange.isEmpty()) {
+        mLoop->exit(0);
+    }
 }
 
-void TestConnRoster::expectPendingContactsFinished(PendingOperation *op)
+void TestConnRoster::expectBlockStatusChanged(bool blocked)
 {
-    if (!op->isFinished()) {
-        qWarning() << "unfinished";
-        mLoop->exit(1);
-        return;
+    Q_UNUSED(blocked);
+
+    Contact *c = qobject_cast<Contact*>(sender());
+    QVERIFY(c);
+
+    ContactPtr contact(c);
+    mContactsExpectingBlockStatusChange.remove(contact->id());
+
+    if (mContactsExpectingBlockStatusChange.isEmpty() && mBlockingContactsFinished) {
+        mLoop->exit(0);
+    }
+}
+
+// This connects to allKnownContactsChanged() but it is only used in the last contact blocking test
+void TestConnRoster::expectBlockedContactsChanged(const Tp::Contacts &added,
+        const Tp::Contacts &removed, const Tp::Channel::GroupMemberChangeDetails &details)
+{
+    Q_UNUSED(details);
+
+    Q_FOREACH(const ContactPtr &contact, added) {
+        mContactsExpectingBlockStatusChange.remove(contact->id());
+    }
+    Q_FOREACH(const ContactPtr &contact, removed) {
+        mContactsExpectingBlockStatusChange.remove(contact->id());
     }
 
-    if (op->isError()) {
-        qWarning().nospace() << op->errorName()
-            << ": " << op->errorMessage();
-        mLoop->exit(2);
-        return;
+    if (mContactsExpectingBlockStatusChange.isEmpty() && mBlockingContactsFinished) {
+        mLoop->exit(0);
     }
-
-    if (!op->isValid()) {
-        qWarning() << "inconsistent results";
-        mLoop->exit(3);
-        return;
-    }
-
-    qDebug() << "finished";
-    PendingContacts *pending = qobject_cast<PendingContacts *>(op);
-    mContacts = pending->contacts();
-
-    mLoop->exit(0);
 }
 
 void TestConnRoster::expectAllKnownContactsChanged(const Tp::Contacts& added, const Tp::Contacts& removed,
@@ -105,9 +110,9 @@ void TestConnRoster::expectAllKnownContactsChanged(const Tp::Contacts& added, co
         QCOMPARE(details.message(), QLatin1String("add me now"));
     }
 
-    if (mConn->contactManager()->allKnownContacts().size() != mHowManyKnownContacts) {
-        qWarning() << "Contacts number mismatch! Watched value: " << mHowManyKnownContacts
-                   << "allKnownContacts(): " << mConn->contactManager()->allKnownContacts().size();
+    if (mConn->client()->contactManager()->allKnownContacts().size() != mHowManyKnownContacts) {
+        qWarning() << "Contacts number mismatch! Watched value: " << mHowManyKnownContacts <<
+            "allKnownContacts(): " << mConn->client()->contactManager()->allKnownContacts().size();
         mLoop->exit(1);
     } else {
         mLoop->exit(0);
@@ -138,59 +143,30 @@ void TestConnRoster::initTestCase()
     tp_debug_set_flags("all");
     dbus_g_bus_get(DBUS_BUS_STARTER, 0);
 
-    gchar *name;
-    gchar *connPath;
-    GError *error = 0;
-
-    mConnService = EXAMPLE_CONTACT_LIST_CONNECTION(g_object_new(
+    mConn = new TestConnHelper(this,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create(Contact::FeatureAlias),
             EXAMPLE_TYPE_CONTACT_LIST_CONNECTION,
             "account", "me@example.com",
             "protocol", "contactlist",
             "simulation-delay", 1,
-            NULL));
-    QVERIFY(mConnService != 0);
-    QVERIFY(tp_base_connection_register(TP_BASE_CONNECTION(mConnService),
-                "contacts", &name, &connPath, &error));
-    QVERIFY(error == 0);
-
-    QVERIFY(name != 0);
-    QVERIFY(connPath != 0);
-
-    mConnName = QLatin1String(name);
-    mConnPath = QLatin1String(connPath);
-
-    g_free(name);
-    g_free(connPath);
+            NULL);
+    QCOMPARE(mConn->connect(), true);
 }
 
 void TestConnRoster::init()
 {
     initImpl();
-
-    mConn = Connection::create(mConnName, mConnPath,
-            ChannelFactory::create(QDBusConnection::sessionBus()),
-            ContactFactory::create(Contact::FeatureAlias));
-
-    QVERIFY(connect(mConn->lowlevel()->requestConnect(),
-                    SIGNAL(finished(Tp::PendingOperation*)),
-                    SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
-    QCOMPARE(mConn->isReady(), true);
-    QCOMPARE(mConn->status(), ConnectionStatusConnected);
 }
 
 void TestConnRoster::testRoster()
 {
     Features features = Features() << Connection::FeatureRoster;
-    QVERIFY(connect(mConn->becomeReady(features),
-            SIGNAL(finished(Tp::PendingOperation*)),
-            this,
-            SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
-    QCOMPARE(mConn->isReady(features), true);
+    QCOMPARE(mConn->enableFeatures(features), true);
 
-    QCOMPARE(static_cast<uint>(mConn->contactManager()->state()),
-             static_cast<uint>(ContactListStateSuccess));
+    ContactManagerPtr contactManager = mConn->client()->contactManager();
+
+    QCOMPARE(contactManager->state(), ContactListStateSuccess);
 
     QStringList toCheck = QStringList() <<
         QLatin1String("sjoerd@example.com") <<
@@ -200,12 +176,13 @@ void TestConnRoster::testRoster()
         QLatin1String("helen@example.com") <<
         QLatin1String("geraldine@example.com") <<
         QLatin1String("guillaume@example.com") <<
-        QLatin1String("christian@example.com");
+        QLatin1String("christian@example.com") <<
+        QLatin1String("bill@example.com") <<
+        QLatin1String("steve@example.com");
     QStringList ids;
     QList<ContactPtr> pendingSubscription;
     QList<ContactPtr> pendingPublish;
-    Q_FOREACH (const ContactPtr &contact,
-            mConn->contactManager()->allKnownContacts()) {
+    Q_FOREACH (const ContactPtr &contact, contactManager->allKnownContacts()) {
         QVERIFY(contact->requestedFeatures().contains(Contact::FeatureAlias));
         qDebug() << " contact:" << contact->id() <<
             "- subscription:" << contact->subscriptionState() <<
@@ -226,18 +203,14 @@ void TestConnRoster::testRoster()
     // Wait for the contacts to be built
     ids = QStringList() << QString(QLatin1String("john@example.com"))
         << QString(QLatin1String("mary@example.com"));
-    QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(ids),
-                    SIGNAL(finished(Tp::PendingOperation*)),
-                    SLOT(expectPendingContactsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
+    QList<ContactPtr> contacts = mConn->contacts(ids);
+    QCOMPARE(contacts.size(), ids.size());
 
-    int i = 0;
-
-    QVERIFY(connect(mConn->contactManager().data(),
+    QVERIFY(connect(contactManager.data(),
                     SIGNAL(presencePublicationRequested(Tp::Contacts,QString)),
                     SLOT(expectPresencePublicationRequested(Tp::Contacts))));
-
-    Q_FOREACH (const ContactPtr &contact, mContacts) {
+    int i = 0;
+    Q_FOREACH (const ContactPtr &contact, contacts) {
         mGotPresenceStateChanged = false;
         mGotPPR = false;
 
@@ -338,9 +311,9 @@ void TestConnRoster::testRoster()
     // Test allKnownContactsChanged.
     // In this test, everytime a subscription is requested or rejected, allKnownContacts changes
     // Cache the current value
-    mHowManyKnownContacts = mConn->contactManager()->allKnownContacts().size();
+    mHowManyKnownContacts = contactManager->allKnownContacts().size();
     // Watch for contacts changed
-    QVERIFY(connect(mConn->contactManager().data(),
+    QVERIFY(connect(contactManager.data(),
                     SIGNAL(allKnownContactsChanged(Tp::Contacts,Tp::Contacts,
                             Tp::Channel::GroupMemberChangeDetails)),
                     SLOT(expectAllKnownContactsChanged(Tp::Contacts,Tp::Contacts,
@@ -349,46 +322,172 @@ void TestConnRoster::testRoster()
     // Wait for the contacts to be built
     ids = QStringList() << QString(QLatin1String("kctest1@example.com"))
         << QString(QLatin1String("kctest2@example.com"));
-    QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(ids),
-                    SIGNAL(finished(Tp::PendingOperation*)),
-                    SLOT(expectPendingContactsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
-
-    Q_FOREACH (const ContactPtr &contact, mContacts) {
+    contacts = mConn->contacts(ids);
+    QCOMPARE(contacts.size(), ids.size());
+    Q_FOREACH (const ContactPtr &contact, contacts) {
         contact->requestPresenceSubscription(QLatin1String("add me now"));
 
         // allKnownContacts is supposed to change here.
         QCOMPARE(mLoop->exec(), 0);
     }
+
+
+    QVERIFY(disconnect(contactManager.data(),
+                       SIGNAL(allKnownContactsChanged(Tp::Contacts,Tp::Contacts,
+                              Tp::Channel::GroupMemberChangeDetails)),
+                       this,
+                       SLOT(expectAllKnownContactsChanged(Tp::Contacts,Tp::Contacts,
+                            Tp::Channel::GroupMemberChangeDetails))));
+
+    // verify that the CM supports contact blocking
+    QVERIFY(contactManager->canBlockContacts());
+
+    // check if the initially blocked contacts are there
+    ids.clear();
+    toCheck = QStringList() <<
+        QLatin1String("bill@example.com") <<
+        QLatin1String("steve@example.com");
+    Q_FOREACH (const ContactPtr &contact, contactManager->allKnownContacts()) {
+        if (contact->isBlocked()) {
+            qDebug() << "blocked contact:" << contact->id();
+            ids << contact->id();
+        }
+    }
+    ids.sort();
+    toCheck.sort();
+    QCOMPARE(ids, toCheck);
+
+    // block all contacts
+    QList<ContactPtr> contactsList = contactManager->allKnownContacts().toList();
+    QSet<QString> contactIdsList;
+    Q_FOREACH (const ContactPtr &contact, contactsList) {
+        QVERIFY(connect(contact.data(),
+                        SIGNAL(blockStatusChanged(bool)),
+                        SLOT(expectBlockStatusChanged(bool))));
+        contactIdsList.insert(contact->id());
+    }
+
+    mBlockingContactsFinished = false;
+    mContactsExpectingBlockStatusChange = contactIdsList;
+
+    // those are already blocked; do not expect their status to change
+    mContactsExpectingBlockStatusChange.remove(QLatin1String("bill@example.com"));
+    mContactsExpectingBlockStatusChange.remove(QLatin1String("steve@example.com"));
+
+    QVERIFY(connect(contactManager->blockContacts(contactsList),
+                    SIGNAL(finished(Tp::PendingOperation*)),
+                    SLOT(expectBlockingContactsFinished(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
+
+    // verify all contacts have been blocked
+    Q_FOREACH (const ContactPtr &contact, contactsList) {
+        QCOMPARE(contact->isBlocked(), true);
+        QVERIFY(contactManager->allKnownContacts().contains(contact));
+    }
+
+    // now remove kctest1 from the server
+    ContactPtr kctest1;
+    Q_FOREACH (const ContactPtr &contact, contactsList) {
+        if (contact->id() == QLatin1String("kctest1@example.com")) {
+            kctest1 = contact;
+        }
+    }
+    QVERIFY(!kctest1.isNull());
+    QVERIFY(connect(contactManager->removeContacts(QList<ContactPtr>() << kctest1),
+                    SIGNAL(finished(Tp::PendingOperation*)),
+                    mLoop, SLOT(quit())));
+    QCOMPARE(mLoop->exec(), 0);
+
+    // allKnownContacts must still contain kctest1, since it is in the deny list
+    QVERIFY(contactManager->allKnownContacts().contains(kctest1));
+    kctest1.reset(); //no longer needed
+
+    // unblock all contacts
+    mBlockingContactsFinished = false;
+    mContactsExpectingBlockStatusChange = contactIdsList;
+
+    QVERIFY(connect(contactManager->unblockContacts(contactsList),
+                    SIGNAL(finished(Tp::PendingOperation*)),
+                    SLOT(expectBlockingContactsFinished(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
+
+    // verify all contacts have been unblocked
+    Q_FOREACH (const ContactPtr &contact, contactsList) {
+        QCOMPARE(contact->isBlocked(), false);
+
+        // ...and that bill, steve and kctest1 have also been removed from allKnownContacts()
+        // note: allKnownContacts() changes here because bill, steve and kctest,
+        // which were only in the deny list, do not exist in any other list, so
+        // they are removed as soon as they get unblocked.
+        if (contact->id() == QLatin1String("bill@example.com") ||
+            contact->id() == QLatin1String("steve@example.com") ||
+            contact->id() == QLatin1String("kctest1@example.com")) {
+            QVERIFY(!contactManager->allKnownContacts().contains(contact));
+        } else {
+            QVERIFY(contactManager->allKnownContacts().contains(contact));
+        }
+    }
+
+    // block some contacts that are not already known
+    ids = QStringList() <<
+        QLatin1String("blocktest1@example.com") <<
+        QLatin1String("blocktest2@example.com");
+    contacts = mConn->contacts(ids);
+
+    // Watch changes in allKnownContacts() instead of watching the Contacts' block status
+    // as we want to destroy the Contact objects and verify that they are being re-created correctly
+    QVERIFY(connect(contactManager.data(),
+                    SIGNAL(allKnownContactsChanged(Tp::Contacts,Tp::Contacts,
+                            Tp::Channel::GroupMemberChangeDetails)),
+                    SLOT(expectBlockedContactsChanged(Tp::Contacts,Tp::Contacts,
+                            Tp::Channel::GroupMemberChangeDetails))));
+
+    mBlockingContactsFinished = false;
+    mContactsExpectingBlockStatusChange = ids.toSet();
+
+    QVERIFY(connect(contactManager->blockContacts(contacts),
+                    SIGNAL(finished(Tp::PendingOperation*)),
+                    SLOT(expectBlockingContactsFinished(Tp::PendingOperation*))));
+
+    // destroy the Contact objects to let them be re-created when the block operation finishes
+    contacts.clear();
+    QCOMPARE(mLoop->exec(), 0);
+
+    // construct the same contacts again and verify that they are blocked
+    contacts = mConn->contacts(ids);
+    Q_FOREACH (const ContactPtr &contact, contacts) {
+        QCOMPARE(contact->isBlocked(), true);
+        QVERIFY(contactManager->allKnownContacts().contains(contact));
+    }
+
+    // now unblock them again
+    mBlockingContactsFinished = false;
+    mContactsExpectingBlockStatusChange = ids.toSet();
+
+    QVERIFY(connect(contactManager->unblockContacts(contacts),
+            SIGNAL(finished(Tp::PendingOperation*)),
+            SLOT(expectBlockingContactsFinished(Tp::PendingOperation*))));
+
+    // note: allKnownContacts() is expected to change again, so we expect
+    // to quit from expectBlockedContactsChanged()
+    QCOMPARE(mLoop->exec(), 0);
+
+    // and verify that they are not in allKnownContacts()
+    Q_FOREACH (const ContactPtr &contact, contacts) {
+        QCOMPARE(contact->isBlocked(), false);
+        QVERIFY(!contactManager->allKnownContacts().contains(contact));
+    }
 }
 
 void TestConnRoster::cleanup()
 {
-    if (mConn) {
-        // Disconnect and wait for the readiness change
-        QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
-                        SIGNAL(finished(Tp::PendingOperation*)),
-                        SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-        QCOMPARE(mLoop->exec(), 0);
-
-        if (mConn->isValid()) {
-            QVERIFY(connect(mConn.data(),
-                            SIGNAL(invalidated(Tp::DBusProxy *,
-                                               const QString &, const QString &)),
-                            SLOT(expectConnInvalidated())));
-            QCOMPARE(mLoop->exec(), 0);
-        }
-    }
-
     cleanupImpl();
 }
 
 void TestConnRoster::cleanupTestCase()
 {
-    if (mConnService != 0) {
-        g_object_unref(mConnService);
-        mConnService = 0;
-    }
+    QCOMPARE(mConn->disconnect(), true);
+    delete mConn;
 
     cleanupTestCaseImpl();
 }
